@@ -24,8 +24,13 @@
 #include <QAbstractItemView>
 #include <QCheckBox>
 #include <QCompleter>
+#include <QDateTime>
 #include <QDialog>
 #include <QDialogButtonBox>
+#include <QDir>
+#include <QDirIterator>
+#include <QFile>
+#include <QFileInfo>
 #include <QFormLayout>
 #include <QHBoxLayout>
 #include <QLineEdit>
@@ -38,8 +43,70 @@
 #include <QUrlQuery>
 #include <QVBoxLayout>
 #include <QWebEngineCookieStore>
+#include <QWebEngineProfile>
 #include <QWebEngineScript>
 #include <QWebEngineView>
+#include <QStandardPaths>
+
+namespace {
+qint64 directorySize(const QString &path)
+{
+    QDir dir(path);
+    if (!dir.exists()) {
+        return -1;
+    }
+    qint64 total = 0;
+    QDirIterator it(path, QDir::Files | QDir::NoSymLinks, QDirIterator::Subdirectories);
+    while (it.hasNext()) {
+        it.next();
+        total += it.fileInfo().size();
+    }
+    return total;
+}
+
+qint64 totalDirectorySize(const QStringList &paths)
+{
+    qint64 total = 0;
+    bool found = false;
+    for (const auto &path : paths) {
+        if (!QDir(path).exists()) {
+            continue;
+        }
+        const qint64 size = directorySize(path);
+        if (size >= 0) {
+            total += size;
+            found = true;
+        }
+    }
+    return found ? total : -1;
+}
+
+QStringList collectCachePaths(const QWebEngineProfile *profile)
+{
+    if (!profile) return {};
+    const QString cachePath = profile->cachePath();
+    if (cachePath.isEmpty()) return {};
+    QDir dir(cachePath);
+    QStringList subdirs;
+    for (const QString &entry : dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot)) {
+        subdirs << cachePath + "/" + entry;
+    }
+    return subdirs;
+}
+
+bool removeCachePath(const QString &path)
+{
+    QFileInfo info(path);
+    if (!info.exists()) {
+        return false;
+    }
+    if (info.isFile() || info.isSymLink()) {
+        return QFile::remove(path);
+    }
+    QDir dir(path);
+    return dir.removeRecursively();
+}
+} // namespace
 
 MainWindow::MainWindow(const QCommandLineParser &argParser, QWidget *parent)
     : QMainWindow(parent),
@@ -47,7 +114,8 @@ MainWindow::MainWindow(const QCommandLineParser &argParser, QWidget *parent)
       searchBox {new QLineEdit(this)},
       progressBar {new QProgressBar(this)},
       toolBar {new QToolBar(this)},
-      tabWidget {new TabWidget(this)},
+      webProfile {new QWebEngineProfile("mx-viewer", this)},
+      tabWidget {new TabWidget(webProfile, this)},
       args {&argParser}
 {
     init();
@@ -74,7 +142,8 @@ MainWindow::MainWindow(const QUrl &url, QWidget *parent)
       searchBox {new QLineEdit(this)},
       progressBar {new QProgressBar(this)},
       toolBar {new QToolBar(this)},
-      tabWidget {new TabWidget(this)},
+      webProfile {new QWebEngineProfile("mx-viewer", this)},
+      tabWidget {new TabWidget(webProfile, this)},
       args {nullptr}
 {
     init();
@@ -110,9 +179,7 @@ void MainWindow::init()
             closedTabs.append({url, icon});
         }
     });
-    if (auto *webView = currentWebView()) {
-        websettings = webView->settings();
-    }
+    websettings = webProfile->settings();
     loadSettings();
     addToolbar();
     addActions();
@@ -360,6 +427,7 @@ QString MainWindow::buildHistoryPageHtml()
 <html>
 <head>
   <meta charset="utf-8">
+  <meta http-equiv="cache-control" content="no-cache">
   <title>%1</title>
   <style>
     :root { color-scheme: light; }
@@ -577,12 +645,15 @@ void MainWindow::addNavigationActions()
     auto *stop = pageAction(QWebEnginePage::Stop);
     toolBar->addAction(back);
     toolBar->addAction(forward);
-    toolBar->addAction(reload);
+    reloadAction = new QAction(reload->icon(), reload->text(), this);
+    reloadAction->setShortcutContext(Qt::ApplicationShortcut);
+    toolBar->addAction(reloadAction);
     toolBar->addAction(stop);
     back->setShortcut(QKeySequence::Back);
     forward->setShortcut(QKeySequence::Forward);
-    reload->setShortcuts(QKeySequence::Refresh);
+    reloadAction->setShortcuts(QKeySequence::Refresh);
     stop->setShortcut(QKeySequence::Cancel);
+    connect(reloadAction, &QAction::triggered, this, &MainWindow::reloadCurrentView);
     connect(stop, &QAction::triggered, this, [this] { done(true); });
 }
 
@@ -796,13 +867,14 @@ void MainWindow::loadSettings()
     // Load first from system .conf file and then overwrite with CLI switches where available
     websettings->setAttribute(QWebEngineSettings::FullScreenSupportEnabled, true);
     websettings->setAttribute(QWebEngineSettings::DnsPrefetchEnabled, true);
-    QWebEngineProfile::defaultProfile()->setHttpAcceptLanguage(QLocale::system().name());
+    webProfile->setHttpAcceptLanguage(QLocale::system().name());
 
     homeAddress = settings.value("Home", "https://start.duckduckgo.com").toString();
     showProgress = settings.value("ShowProgressBar", false).toBool();
     openNewTabWithHome = settings.value("OpenNewTabWithHome", true).toBool();
     zoomPercent = settings.value("ZoomPercent", 100).toInt();
     cookiesEnabled = settings.value("EnableCookies", true).toBool();
+    clearCookiesAtExit = settings.value("ClearCookiesAtExit", false).toBool();
     searchEngine = settings.value("SearchEngine", "DuckDuckGo").toString();
     searchEngineCustom = settings.value("SearchEngineCustom", QString()).toString();
     if (!settings.contains("EnableJavaScript") && settings.contains("DisableJava")) {
@@ -982,7 +1054,7 @@ void MainWindow::setConnections()
         disconnect(urlChangedConn);
     }
     urlChangedConn = connect(currentWebView(), &QWebEngineView::urlChanged, this, &MainWindow::updateUrl);
-    connect(QWebEngineProfile::defaultProfile(), &QWebEngineProfile::downloadRequested, downloadWidget,
+    connect(webProfile, &QWebEngineProfile::downloadRequested, downloadWidget,
             &DownloadWidget::downloadRequested, Qt::UniqueConnection);
     if (loadFinishedConn) {
         disconnect(loadFinishedConn);
@@ -1042,7 +1114,7 @@ void MainWindow::tabChanged()
     auto *forward = pageAction(QWebEnginePage::Forward);
     auto *reload = pageAction(QWebEnginePage::Reload);
     auto *stop = pageAction(QWebEnginePage::Stop);
-    QMap<QString, QAction *> actionMap = {{"Back", back}, {"Forward", forward}, {"Reload", reload}, {"Stop", stop}};
+    QMap<QString, QAction *> actionMap = {{"Back", back}, {"Forward", forward}, {"Reload", reloadAction}, {"Stop", stop}};
     auto actionList = toolBar->actions();
     toolBar->setUpdatesEnabled(false);
     for (int i = 0; i < actionList.size() - 1; ++i) {
@@ -1056,6 +1128,12 @@ void MainWindow::tabChanged()
         }
     }
     toolBar->setUpdatesEnabled(true);
+    if (reloadAction) {
+        reloadAction->setIcon(reload->icon());
+        reloadAction->setText(reload->text());
+        reloadAction->setToolTip(reload->toolTip());
+        reloadAction->setEnabled(reload->isEnabled());
+    }
     addressBar->setText(currentWebView()->url().toString());
     if (addressBar->text().isEmpty()) {
         addressBar->setFocus();
@@ -1235,14 +1313,30 @@ QString MainWindow::buildSettingsPageHtml()
     const bool enableThirdPartyCookies = settings.value("EnableThirdPartyCookies", true).toBool();
     const bool allowPopups = settings.value("AllowPopups", true).toBool();
     const bool saveTabs = settings.value("SaveTabs", false).toBool();
+    const bool clearCookiesAtExit = settings.value("ClearCookiesAtExit", false).toBool();
 
     auto check = [](bool value) { return value ? QStringLiteral("checked") : QString(); };
     auto disabled = [](bool value) { return value ? QString() : QStringLiteral("disabled"); };
 
+    QString cacheSizeText = clearingCache ? tr("Clearing...") : tr("unknown");
+    if (!clearingCache) {
+        const auto *profile = webProfile;
+        const QStringList cacheCandidates = collectCachePaths(profile);
+        const qint64 cacheBytes = totalDirectorySize(cacheCandidates);
+        if (cacheBytes >= 0) {
+            if (cacheBytes < 1024) {
+                cacheSizeText = tr("Cleared");
+            } else {
+                cacheSizeText = DownloadWidget::withUnit(cacheBytes);
+            }
+        }
+    }
+
     const QString html = QStringLiteral(R"(<!doctype html>
 <html>
 <head>
   <meta charset="utf-8">
+  <meta http-equiv="cache-control" content="no-cache">
   <title>%1</title>
   <style>
     :root { color-scheme: light; }
@@ -1253,8 +1347,11 @@ QString MainWindow::buildSettingsPageHtml()
     .row { display: grid; gap: 6px; }
     .input, select { padding: 8px 10px; border: 1px solid #d0d7de; border-radius: 6px; }
     .check { display: flex; gap: 8px; align-items: center; }
+    .check-row { display: flex; gap: 12px; align-items: center; }
+    .cache-label { font-weight: 600; }
     .actions { display: flex; gap: 12px; margin-top: 8px; }
     .btn { padding: 8px 12px; border: 1px solid #d0d7de; background: #f6f8fa; border-radius: 6px; cursor: pointer; }
+    .btn-inline { padding: 4px 8px; font-size: 12px; }
   </style>
 </head>
 <body>
@@ -1286,13 +1383,21 @@ QString MainWindow::buildSettingsPageHtml()
     <label class="check"><input id="spatialNav" type="checkbox" %21> %22</label>
     <label class="check"><input id="enableJs" type="checkbox" %23> %24</label>
     <label class="check"><input id="loadImages" type="checkbox" %25> %26</label>
-    <label class="check"><input id="enableCookies" type="checkbox" %27> %28</label>
+    <div class="check-row">
+      <label class="check"><input id="enableCookies" type="checkbox" %27> %28</label>
+      <button class="btn btn-inline" id="clearCookies" type="button">%41</button>
+    </div>
     <label class="check"><input id="thirdPartyCookies" type="checkbox" %29 %30> %31</label>
-    <label class="check"><input id="allowPopups" type="checkbox" %32> %33</label>
-    <label class="check"><input id="saveTabs" type="checkbox" %34> %35</label>
+    <label class="check"><input id="clearCookiesAtExit" type="checkbox" %32> %33</label>
+    <label class="check"><input id="allowPopups" type="checkbox" %34> %35</label>
+    <label class="check"><input id="saveTabs" type="checkbox" %36> %37</label>
+    <div class="check-row">
+      <div class="cache-label">%42</div>
+      <button class="btn btn-inline" id="clearCache" type="button">%43</button>
+    </div>
     <div class="actions">
-      <button class="btn" id="save">%36</button>
-      <button class="btn" id="reset" type="button">%37</button>
+      <button class="btn" id="save">%38</button>
+      <button class="btn" id="reset" type="button">%39</button>
     </div>
   </form>
   <script>
@@ -1301,12 +1406,33 @@ QString MainWindow::buildSettingsPageHtml()
     const resetBtn = document.getElementById('reset');
     const searchSelect = document.getElementById('search');
     const customSearch = document.getElementById('customSearch');
+    const inputs = Array.from(form.querySelectorAll('input, select'));
+    saveBtn.disabled = true;
+    resetBtn.disabled = true;
+    window.mxSettingsDirty = false;
+    function snapshot() {
+      const data = {};
+      inputs.forEach(el => {
+        if (!el.id) {
+          return;
+        }
+        data[el.id] = el.type === 'checkbox' ? el.checked : el.value;
+      });
+      return JSON.stringify(data);
+    }
+    let baseline = snapshot();
+    function updateDirtyState() {
+      const dirty = snapshot() !== baseline;
+      window.mxSettingsDirty = dirty;
+      saveBtn.disabled = !dirty;
+      resetBtn.disabled = !dirty;
+    }
     function boolValue(id) {
       return document.getElementById(id).checked ? '1' : '0';
     }
     function normalizeCustomUrl(url) {
       if (!url.includes('%s') && !url.includes('?q=')) {
-        const add = confirm('%38');
+        const add = confirm('%40');
         if (add) {
           const join = url.includes('?') ? '&' : '?';
           return url + join + 'q=%s';
@@ -1321,6 +1447,9 @@ QString MainWindow::buildSettingsPageHtml()
     }
     saveBtn.addEventListener('click', event => {
       event.preventDefault();
+      if (saveBtn.disabled) {
+        return;
+      }
       let customUrl = customSearch.value.trim();
       if (searchSelect.value === 'Custom' && customUrl.length > 0) {
         customUrl = normalizeCustomUrl(customUrl);
@@ -1340,13 +1469,34 @@ QString MainWindow::buildSettingsPageHtml()
       params.set('thirdPartyCookies', boolValue('thirdPartyCookies'));
       params.set('allowPopups', boolValue('allowPopups'));
       params.set('saveTabs', boolValue('saveTabs'));
+      params.set('clearCookiesAtExit', boolValue('clearCookiesAtExit'));
+      baseline = snapshot();
+      updateDirtyState();
       location.href = 'mx-settings://save?' + params.toString();
     });
     resetBtn.addEventListener('click', event => {
       event.preventDefault();
       location.href = 'mx-settings://list';
     });
+    const clearCookiesBtn = document.getElementById('clearCookies');
+    clearCookiesBtn.addEventListener('click', event => {
+      event.preventDefault();
+      if (confirm('%44')) {
+        location.href = 'mx-settings://clearCookies';
+      }
+    });
+    const clearCacheBtn = document.getElementById('clearCache');
+    clearCacheBtn.addEventListener('click', event => {
+      event.preventDefault();
+      if (confirm('%45')) {
+        location.href = 'mx-settings://clearCache';
+      }
+    });
     searchSelect.addEventListener('change', syncCustom);
+    inputs.forEach(el => {
+      el.addEventListener('input', updateDirtyState);
+      el.addEventListener('change', updateDirtyState);
+    });
     const cookiesToggle = document.getElementById('enableCookies');
     const thirdPartyToggle = document.getElementById('thirdPartyCookies');
     function syncThirdParty() {
@@ -1358,6 +1508,8 @@ QString MainWindow::buildSettingsPageHtml()
     syncCustom();
     cookiesToggle.addEventListener('change', syncThirdParty);
     syncThirdParty();
+    baseline = snapshot();
+    updateDirtyState();
   </script>
 </body>
 </html>)")
@@ -1392,6 +1544,8 @@ QString MainWindow::buildSettingsPageHtml()
                                  check(enableThirdPartyCookies),
                                  disabled(enableCookies),
                                  tr("Enable third-party cookies").toHtmlEscaped(),
+                                 check(clearCookiesAtExit),
+                                 tr("Clear cookies at exit").toHtmlEscaped(),
                                  check(allowPopups),
                                  tr("Allow pop-up windows").toHtmlEscaped(),
                                  check(saveTabs),
@@ -1399,7 +1553,12 @@ QString MainWindow::buildSettingsPageHtml()
                                  tr("Save settings").toHtmlEscaped(),
                                  tr("Reset").toHtmlEscaped(),
                                  tr("Custom URL has no ?q= or %s placeholder. Append ?q=%s automatically?")
-                                     .toHtmlEscaped());
+                                     .toHtmlEscaped(),
+                                 tr("Clear cookies").toHtmlEscaped(),
+                                 tr("Cache size: %1").arg(cacheSizeText).toHtmlEscaped(),
+                                 tr("Clear cache").toHtmlEscaped(),
+                                 tr("Clear all cookies?").toHtmlEscaped(),
+                                 tr("Clear the cache?").toHtmlEscaped());
 
     return html;
 }
@@ -1409,7 +1568,8 @@ void MainWindow::renderSettingsPage(WebView *view)
     if (!view) {
         return;
     }
-    view->setHtml(buildSettingsPageHtml(), QUrl("mx-settings://list"));
+    const QString ts = QString::number(QDateTime::currentMSecsSinceEpoch());
+    view->setHtml(buildSettingsPageHtml(), QUrl("mx-settings://list?ts=" + ts));
     view->show();
     tabWidget->setTabText(tabWidget->indexOf(view), tr("Settings"));
     setWindowTitle(tr("Settings"));
@@ -1438,9 +1598,35 @@ bool MainWindow::handleSettingsRequest(const QUrl &url)
         return false;
     }
     const QString action = url.host();
-    if (action != "save") {
+    if (action == "clearcookies") {
+        webProfile->cookieStore()->deleteAllCookies();
         renderSettingsPage(currentWebView());
         return true;
+    } else if (action == "clearcache") {
+        clearingCache = true;
+        renderSettingsPage(currentWebView());
+        webProfile->clearHttpCache();
+#if QT_VERSION >= QT_VERSION_CHECK(6, 7, 0)
+        connect(webProfile, &QWebEngineProfile::clearHttpCacheCompleted, this, [this]() {
+            clearingCache = false;
+            if (auto *view = currentWebView()) {
+                if (view->url().scheme() == "mx-settings") {
+                    renderSettingsPage(view);
+                }
+            }
+        }, Qt::SingleShotConnection);
+#else
+        // Qt < 6.7 doesn't have clearHttpCacheCompleted signal, use timer fallback
+        QTimer::singleShot(500, this, [this]() {
+            clearingCache = false;
+            if (auto *view = currentWebView()) {
+                if (view->url().scheme() == "mx-settings") {
+                    renderSettingsPage(view);
+                }
+            }
+        });
+#endif
+        return true;
     }
     QUrlQuery query(url);
     const QString newHome = QUrl::fromPercentEncoding(query.queryItemValue("home").toUtf8()).trimmed();
@@ -1456,6 +1642,7 @@ bool MainWindow::handleSettingsRequest(const QUrl &url)
     bool newThirdParty = query.queryItemValue("thirdPartyCookies") == "1";
     const bool newAllowPopups = query.queryItemValue("allowPopups") == "1";
     const bool newSaveTabs = query.queryItemValue("saveTabs") == "1";
+    const bool newClearCookiesAtExit = query.queryItemValue("clearCookiesAtExit") == "1";
     if (!newEnableCookies) {
         newThirdParty = false;
     }
@@ -1479,6 +1666,8 @@ bool MainWindow::handleSettingsRequest(const QUrl &url)
     settings.setValue("EnableThirdPartyCookies", newThirdParty);
     settings.setValue("AllowPopups", newAllowPopups);
     settings.setValue("SaveTabs", newSaveTabs);
+    clearCookiesAtExit = newClearCookiesAtExit;
+    settings.setValue("ClearCookiesAtExit", newClearCookiesAtExit);
     if (newZoom > 0) {
         setZoomPercent(newZoom, true);
     }
@@ -1513,7 +1702,7 @@ void MainWindow::applyWebSettings()
     websettings->setAttribute(QWebEngineSettings::LocalStorageEnabled, enableCookies);
     websettings->setAttribute(QWebEngineSettings::JavascriptCanOpenWindows, allowPopups);
 
-    auto *profile = QWebEngineProfile::defaultProfile();
+    auto *profile = webProfile;
     profile->setHttpAcceptLanguage(QLocale::system().name());
 
     if (cookiesEnabled && !enableCookies) {
@@ -1884,9 +2073,7 @@ void MainWindow::keyPressEvent(QKeyEvent *event)
         return;
     }
     if (event->key() == Qt::Key_R && event->modifiers() == Qt::ControlModifier) {
-        if (auto *view = currentWebView()) {
-            view->reload();
-        }
+        reloadCurrentView();
         return;
     }
     if (event->key() == Qt::Key_Left && event->modifiers() == Qt::AltModifier) {
@@ -1914,6 +2101,9 @@ void MainWindow::resizeEvent(QResizeEvent * /*event*/)
 void MainWindow::closeEvent(QCloseEvent * /*event*/)
 {
     downloadWidget->close();
+    if (clearCookiesAtExit) {
+        webProfile->cookieStore()->deleteAllCookies();
+    }
     settings.setValue("Geometry", saveGeometry());
 
     if (settings.value("SaveTabs", false).toBool()) {
@@ -1939,6 +2129,19 @@ WebView *MainWindow::currentWebView()
     return tabWidget->currentWebView();
 }
 
+void MainWindow::reloadCurrentView()
+{
+    auto *view = currentWebView();
+    if (!view) {
+        return;
+    }
+    if (view->url().scheme() == "mx-settings") {
+        renderSettingsPage(view);
+        return;
+    }
+    view->reload();
+}
+
 // display progressbar while loading page
 void MainWindow::loading()
 {
